/// <reference types="knockout" />
import "tocca";
import { Style, StyleObservable } from "./style";
import { AttributeName } from "./types";
import { IEventListener, IAddEventListenerOptions, UiElementEventMap } from "./eventListener";
import * as types from "./types";
export declare const settings: {
    tapDelay: number;
};
export declare function setObserverOptions(options: MutationObserverInit): void;
export interface Attribute {
    name: AttributeName;
    value: string | number | KnockoutObservable<string | number>;
}
export interface IUiElement {
    render(): HTMLElement;
    getElement(): HTMLElement;
}
export interface UiElementOptions {
    id?: string | number;
    elementType?: string | KnockoutObservable<string>;
    classNames?: string[] | KnockoutObservableArray<string>;
    attributes?: Array<Attribute> | KnockoutObservableArray<Attribute>;
    eventListeners?: Array<IEventListener>;
    style?: Style | StyleObservable;
    innerHtml?: string | KnockoutObservable<string>;
    innerText?: string | KnockoutObservable<string>;
    children?: Array<IUiElement> | KnockoutObservableArray<IUiElement>;
    addControlToDataDictionary?: boolean;
    userSelectable?: boolean;
    dispose?: () => void;
    disposeOnDomRemoval?: boolean;
    refresh?: () => void;
    visible?: boolean | KnockoutObservable<boolean>;
}
export declare abstract class UiElement<TOptions extends UiElementOptions = UiElementOptions> implements IUiElement {
    protected constructor(tagName: string, elementType: string, options?: TOptions);
    private _tagName;
    private _elementType;
    private _attributes;
    private _classes;
    private _eventListeners;
    private _style;
    private _userSelectable;
    private _innerHtml;
    private _innerText;
    private _addControlToDataDictionary;
    private _children;
    private _knockoutSubscriptions;
    private _addEventListenerToElement;
    private _setAttributes;
    private _styleDisplayVisible;
    private _visible;
    protected element: HTMLElement;
    protected build(): void;
    protected readonly options: TOptions;
    render(): HTMLElement;
    refresh(): void;
    remove(): void;
    handleMessage(message: object): void;
    getElement(): HTMLElement;
    focus(options?: FocusOptions): void;
    focusPreviousSibling(options?: FocusOptions): void;
    focusNextSibling(options?: FocusOptions): void;
    setElementType(elementType: any): void;
    setClasses(classNames: string[], replace: boolean): void;
    removeClasses(classNames: string[]): void;
    setInnerHtml(innerHtml: string): void;
    setInnerText(innerText: string): void;
    setAttribute(name: types.AttributeName, value: string | number): void;
    removeAttribute(attributeName: string): void;
    getAttribute(attributeName: types.AttributeName): string | number;
    getStyle(...cssProperties: Array<types.CssProperty>): Style;
    getStyleValue(cssProperty: types.CssProperty): any;
    setStyle(newStyle: Style, overwriteExisting?: boolean): void;
    setStyleNonStandard(name: string, value: any, overwriteExisting?: boolean): void;
    deleteStyle(style: types.CssProperty | Array<types.CssProperty> | Style): void;
    setVisibility(visible: boolean): void;
    addSubscription<TObservable>(observable: KnockoutObservable<TObservable>, callback: (newValue: TObservable) => void): void;
    dispatchEvent<TType extends keyof UiElementEventMap>(type: TType): boolean;
    addEventListener<TType extends keyof UiElementEventMap>(type: TType, listener: (this: UiElement, event: UiElementEventMap[TType]) => any, options?: IAddEventListenerOptions): void;
    getChildren(): Array<IUiElement>;
    setChildren(children: Array<IUiElement>, replace?: boolean): void;
    addChild(newChild: IUiElement, referenceChild?: IUiElement): void;
    removeChild(element: IUiElement): void;
    empty(): void;
    readonly diposeOnDomRemoval: boolean;
    readonly dispose: () => void;
}
